﻿using ProjectCleaningCar.CollectionGenericObjects;
using ProjectCleaningCar.Drawning;

namespace ProjectCleaningCar;
/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>
public partial class FormCleaningCarCollection : Form
{
    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;
    /// <summary>
    /// Конструктор
    /// </summary>
    public FormCleaningCarCollection()
    {
        InitializeComponent();
    }
    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        switch (comboBoxSelectorCompany.Text)
        {
            case "Автопарк":
                _company = new AutoParkService(pictureBox.Width,
                pictureBox.Height, new MassiveGenericObjects<DrawningTruck>());
                break;
        }
    }
    /// <summary>
    /// Добавление грузовика
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddTruck_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningTruck));
    /// <summary>
    /// Добавление подметательно-уборочной машины
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddCleaningCar_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningCleaningCar));
    /// <summary>
    /// Создание объекта класса-перемещенияв
    /// </summary>
    /// <param name="type"></param>
    private void CreateObject(string type)
    {
        if (_company == null)
        {
            return;
        }
        Random random = new();
        DrawningTruck drawningTruck;
        switch (type)
        {
            case nameof(DrawningTruck):
                drawningTruck = new DrawningTruck(random.Next(100, 300),
                random.Next(1000, 3000), GetColor(random));
                break;
            case nameof(DrawningCleaningCar):
                // TODO вызов диалогового окна для выбора цвета
                drawningTruck = new DrawningCleaningCar(random.Next(100, 300), random.Next(1000, 3000),
                GetColor(random), GetColor(random),
                Convert.ToBoolean(random.Next(0, 2)),
                Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }
        if (_company + drawningTruck != -1)
        {
            MessageBox.Show("Объект добавлен");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось добавить объект");
        }
    }
    /// <summary>
    /// Получение цвета
    /// </summary>
    /// <param name="random">Генератор случайных чисел</param>
    /// <returns></returns>
    private static Color GetColor(Random random)
    {
        Color color = Color.FromArgb(random.Next(0, 256), random.Next(0,
        256), random.Next(0, 256));
        ColorDialog dialog = new();
        if (dialog.ShowDialog() == DialogResult.OK)
        {
            color = dialog.Color;
        }
        return color;
    }
    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRemoveCar_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
        {
            return;
        }
        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }
        int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
        if (_company - pos != null)
        {
            MessageBox.Show("Объект удален");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось удалить объект");
        }

    }
    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        DrawningTruck? truck = null;
        int counter = 100;
        while (truck == null)
        {
            truck = _company.GetRandomObject();
            counter--;
            if (counter <= 0) break;
        }
        if (truck == null)
        {
            return;
        }
        FormCleaningCar form = new FormCleaningCar();
        form.SetCar = truck;
        form.ShowDialog();
    }
    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }
}
